from typing import Any

from ..base import BaseModel


class ServiceParams(BaseModel):
    http_client: Any
    """请求实例"""

    run_id: str
    """连接的唯一 id"""

    user_id: str
    """用户唯一 id"""

    socket: Any
    """socketio实例"""
